from typing import Optional

from tui_rsync.core.ports.orm import DatabaseManagerPort
from tui_rsync.infrastructure.orm.dto.dtos import BackupPlanDTO
from .backup_plan_repository_port import BackupPlanRepositoryPort

from tui_rsync.core.components.backup_plan.domain import BackupPlan
from tui_rsync.core.shared_kernel.components.common import UUID

from tui_rsync.infrastructure.orm.models import BackupPlanModel, DestinationModel
from ..commands import RemoveBackupBackupPlanDestinationsCommand


class BackupPlanRepository(BackupPlanRepositoryPort):
    def __init__(self, database_manager: DatabaseManagerPort):
        self.databaseManager = database_manager

    def add(self, backup_plan: BackupPlan):
        model = BackupPlanDTO.to_model(backup_plan)

        model.save(force_insert=True)
        for destination in model.destinations:
            destination.save(force_insert=True)

    def get_by_id(self, uuid: UUID) -> Optional[BackupPlan]:
        model = BackupPlanModel.get_or_none(BackupPlanModel.id == uuid.id)

        if model is None:
            return model

        return BackupPlanDTO.to_domain(model)

    def update(self, backup_plan: BackupPlan):
        updated_model = BackupPlanDTO.to_model(backup_plan)
        model = BackupPlanModel.get_or_none(BackupPlanModel.id == updated_model.id)
        remove_backup_plan_destinations_command = RemoveBackupBackupPlanDestinationsCommand(self.databaseManager)

        model.label = updated_model.label
        model.source = updated_model.source

        remove_backup_plan_destinations_command.execute(backup_plan.id)

        model.save()
        for destination in updated_model.destinations:
            destination.save(force_insert=True)

    def delete(self, uuid: UUID) -> bool:
        deleted = BackupPlanModel.delete().where(BackupPlanModel.id == uuid.id).execute()
        deleted = DestinationModel.delete().where(DestinationModel.source == uuid.id).execute() | deleted
        return bool(deleted)
