################################################################################
# Copyright (C) 2023-2025 Kostiantyn Klochko <kklochko@protonmail.com>         #
#                                                                              #
# This file is part of tui-rsync.                                              #
#                                                                              #
# tui-rsync is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU General Public License as published by the Free         #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# tui-rsync is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more        #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU General Public License along with #
# tui-rsync. If not, see <https://www.gnu.org/licenses/>.                      #
################################################################################

from rich.console import Console
import typer

from tui_rsync.core.components.backup_plan.application.queries import GetAllBackupBackupPlansQuery

from tui_rsync.core.components.backup_plan.application.services.backup_plan_service import BackupPlanService
from tui_rsync.core.shared_kernel.components.common import UUID
from tui_rsync.infrastructure.configuration import CurrentConfiguration
from tui_rsync.user_interface.cli.components.backup_plan.formating import BackupPlanFormat

console = Console()
show_backup_plan = typer.Typer()


@show_backup_plan.command()
def one(
    id: str = typer.Option(
        None, "--id", "-i",
        help="[b]The label[/] is a uniq identification of a [b]source[/].",
        show_default=False
    ),
):
    """
    [green b]Show[/] an [yellow]existing backup plan by the id[/].
    """

    service: BackupPlanService = CurrentConfiguration.get(BackupPlanService)

    if id is None:
        console.print("[red b][ERROR][/] Backup plan does not exists!!!")
        return

    plan = service.get_by_id(UUID(id))

    if plan is None:
        console.print("[red b][ERROR][/] Backup plan does not exists!!!")
        return

    console.print(BackupPlanFormat.format(plan))


@show_backup_plan.command()
def all():
    """
    [green b]Show[/] [yellow]all existing backup plans[/].
    """

    query = CurrentConfiguration.get(GetAllBackupBackupPlansQuery)

    for backup_plan in query.execute():
        console.print(BackupPlanFormat.format(backup_plan))
